/*
 Alarm System Monitor - Server
 Version: 0.2
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 15/08/2012
 Last Modified: 5/03/2013
 
 The server tracks whether the alarm system is off or has been triggered by checking the Alarm LED. If the LED
 is blinking - the alarm is active, if it's solid on - the alarm is triggered and if it's off - the alarm is off.
 
 Whether the alarm is active or not, we listen for incoming requests from our PIRs. If the alarm has triggered we contact
 our sirens and then go to sleep. Once the alarm has been switched off, we re-contact our sirens and listen for requests again.
 
 */
 
// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//         PB0  2|    |13  PA0 
//         PB1  3|    |12  PA1 
//   Reset PB3  4|    |11  PA2 Trigger Nokia SMS Sender
//      CE PB2  5|    |10  PA3 Alarm LED
//     CSN PA7  6|    |9   PA4 SCK
//      MI PA6  7|    |8   PA5 MO
//               +----+
//

#define F_CPU 1000000 // 1 MHz

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include "hmac-sha1/hmac-sha1.h"
#include "setup.c"
#include "mirf.c"

volatile uint8_t alarm_state = OFF; // Current alarm state, on or off
volatile uint8_t alarm_state_counter = 0; // Incremented every timer0 overflow
volatile uint8_t alarm_siren = OFF; // Is the alarm's siren on or off (has it been triggered)
volatile uint8_t four_second_timeout = false;

// HMAC 256bit random key (you should change this)
uint8_t hmac_key[] = {0x01, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
							0x88, 0x99, 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF,
							0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 
							0x02, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77};

uint8_t data_in[21];
uint8_t data_out[21];

int main(void) {
	setup();
	mirf_init();
	_delay_ms(50);
	mirf_config();
	
	while(1) {
		// Activate external sirens if alarm is triggered
		if (alarm_siren == ON) {
			process_siren_check_in();
		}
		
		RX_POWERUP;
		mirf_flush_rx_tx(); // Flush RX and TX FIFO
		mirf_CE_hi; // Start listening
		
		// Wait for incoming requests from the PIRs as long as the siren is off
		while ((!(mirf_status() & (1<<RX_DR))) && alarm_siren == OFF) { 
			_delay_us(250);
		}
		mirf_CE_lo; // Stop listening
		
		// Process the PIR request as long as the alarm siren is off
		if (alarm_siren == OFF) {
			if (!process_pir_check_in()) {
				for (int x = 0; x < 5; x++) {
					mirf_CSN_lo;
					_delay_ms(50);
					mirf_CSN_hi;
					_delay_ms(50);
				}
			}
		}
	}
}

ISR(WDT_vect) { }

// Pin change means the LED is flashing, alarm is on and the siren is off
// When the LED is solid on, it means the siren is on and when LED is solid off it means the alarm is off
ISR(PCINT0_vect) {
	alarm_state = ON;
	alarm_siren = OFF;
	alarm_state_counter = 0;
	PORTA &= ~(1<<PA2); // Turn off SMS
}

// Check if the alarm state counter has reached 8 after 4 timer overflows (~2 seconds) which means the LED 
// hasn't changed state during this time
ISR(TIM0_OVF_vect) {
	alarm_state_counter++;
	if (alarm_state_counter >= 8) {
		// Alarm LED has been on for 2 seconds (inversed) - disable the siren
		if (PINA & (1<<PA3)) {
			alarm_state = OFF;
			PORTA &= ~(1<<PA2);
		}
		// Alarm LED has been off for 2 seconds (inversed) - turn off the alarm
		else {
			alarm_siren = ON;
			PORTA |= (1<<PA2);
		}
		alarm_state_counter = 0;
	}
}

// Timer overflow after 4 seconds
ISR(TIM1_OVF_vect) {
	four_second_timeout = true;
}