/*
 Alarm System Monitor - Siren
 Version: 0.2
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 15/08/2012
 Last Modified: 5/03/2013
 
 Siren which wirelessly connects to the alarm system. It waits for an incoming request, connects to the server 
 to determine whether the siren needs to be turned on or off.
 
 */
 
// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//         PB0  2|    |13  PA0
//         PB1  3|    |12  PA1
//   Reset PB3  4|    |11  PA2 
//      CE PB2  5|    |10  PA3 Mosfet to turn on Siren
//     CSN PA7  6|    |9   PA4 SCK
//      MI PA6  7|    |8   PA5 MO
//               +----+
//

#define F_CPU 1000000 // 1 MHz

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <string.h>
#include "setup.c"
#include "hmac-sha1/hmac-sha1.h"
#include "mirf.c"

// Current siren state
volatile uint8_t alarm_siren = ALARM_OFF;
int ten_minute_counter = 0;

// 256bit random number (you should change this)
uint8_t random_number[] = {0x27, 0x9B, 0x6D, 0xD8, 0xBA, 0x17, 0x62, 0xD7,
							0xBB, 0x14, 0xF7, 0xDF, 0xDA, 0x41, 0xEE, 0x66,
							0x8A, 0x7A, 0x8D, 0X6F, 0xFA, 0x4F, 0xDA, 0xFC,
							0x41, 0x01, 0x12, 0x3B, 0x9F, 0xF7, 0xF2, 0xAF};

// HMAC 256bit random key (you should change this)
uint8_t hmac_key[] = {0x01, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
							0x88, 0x99, 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF,
							0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 
							0x02, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77};

uint8_t hmac_test[21]; // Used to test the HMAC received from the server
uint8_t data_in[21];
uint8_t data_out[21];

int main(void) {
	setup();
	mirf_init();
	_delay_ms(50);
	mirf_config();
	
	while(1) {
		// Sleep for 1 seconds
		setup_watchdog(T1S);
		system_sleep();
		turnOffwatchdog();
		
		// Switch off the siren after ~10 minutes if we don't hear anything back from the server to turn the siren off
		// 600 seconds / (1 second watchdog + 50ms random block)
		if (ten_minute_counter >= 571) {
			PORTA &= ~(1<<PA3); // Turn off siren
			ten_minute_counter = 0;
		}
		ten_minute_counter++;
		
		// Increment our 256bit random number
		int incrementing = 0;
		for (int x = 31; x >= 0; x--) {
			if (incrementing == 1 && random_number[x] == 0) {
				random_number[x-1]++;
			}
			else if (incrementing == 0) {
				if (random_number[x] == 255) {
					random_number[x]++;
					random_number[x-1]++;
					incrementing = 1;
				}
				else {
					random_number[x]++;
					break;
				}
			}
		}
		
		// Generate the random block (160 bit) using SHA1 from our random number
		sha1(data_out, random_number, 256);
		
		// Set last data_out number to 0 to indicate to the server that we are a siren
		data_out[20] = 1;
		
		// Check in the with server
		if (check_in()) {
			// Perform alarm siren action
			if (alarm_siren == ALARM_ON) {
				PORTA |= (1<<PA3); // Turn on siren
				ten_minute_counter = 0;
			}
			else {
				PORTA &= ~(1<<PA3); // Turn off siren
			}
			
			// Sleep for 2 seconds so other sirens can check in
			setup_watchdog(T2S);
			system_sleep();
			turnOffwatchdog();
		}
		else {
			for (int x = 0; x < 5; x++) {
				mirf_CSN_lo;
				_delay_ms(50);
				mirf_CSN_hi;
				_delay_ms(50);
			}
		}
	}
}

ISR(WDT_vect) { }
